@@ -4,6 +4,8 @@ Reolink Camera API
 import json
 import logging
 from datetime import datetime, timedelta
+from typing import Dict, List, Optional, Tuple
+from . import typings
 
 import asyncio
 import aiohttp
@@ -55,6 +57,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         self._email_state = None
         self._ir_state = None
         self._daynight_state = None
+        self._backlight_state = None
         self._recording_state = None
         self._audio_state = None
         self._rtsp_port = None
@@ -160,6 +163,11 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         """Return the daynight state."""
         return self._daynight_state
 
+    @property
+    def backlight_state(self):
+        """Return the backlight state."""
+        return self._backlight_state
+
     @property
     def recording_state(self):
         """Return the recording state."""
@@ -209,12 +217,12 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
     def protocol(self):
         """Return the protocol."""
         return self._protocol
-    
+
     @property
     def stream_format(self):
         """Return the stream format."""
         return self._stream_format
-    
+
     @property
     def channel(self):
         """Return the channel number."""
@@ -264,6 +272,9 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
         if self._daynight_state is not None:
             capabilities.append("dayNight")
 
+        if self._backlight_state is not None:
+            capabilities.append("backLight")
+
         if self._email_state is not None:
             capabilities.append("email")
 
@@ -402,6 +413,20 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         return stream_source
 
+    async def get_vod_source(self, filename: str):
+        """Return the vod source url."""
+        if not await self.login():
+            return
+
+        """
+        REOLink uses an odd encoding, if the camera provides a / in the filename it needs
+        to be encoded with %20
+        """
+        # VoDs are only available over rtmp, rtsp is not an option
+        stream_source = f"rtmp://{self._host}:{self._rtmp_port}/vod/{filename.replace('/', '%20')}?channel={self._channel}&stream=0&token={self._token}"
+
+        return stream_source
+
     async def map_json_response(self, json_data):  # pylint: disable=too-many-branches
         """Map the JSON objects to internal objects and store for later use."""
         for data in json_data:
@@ -418,7 +443,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                     self._channels = data["value"]["DevInfo"]["channelNum"]
 
                 elif data["cmd"] == "GetHddInfo":
-                    self._hdd_info = data
+                    self._hdd_info = data["value"]["HddInfo"]
 
                 elif data["cmd"] == "GetLocalLink":
                     self._local_link = data
@@ -454,6 +479,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
                 elif data["cmd"] == "GetIsp":
                     self._isp_settings = data
                     self._daynight_state = data["value"]["Isp"]["dayNight"]
+                    self._backlight_state = data["value"]["Isp"]["backLight"]
 
                 elif data["cmd"] == "GetIrLights":
                     self._ir_settings = data
@@ -580,7 +606,7 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
     async def set_protocol(self, protocol):
         """Update the protocol property."""
         self._protocol = protocol
-        
+
     async def set_stream_format(self, stream_format):
         """Update the stream format property."""
         self._stream_format = stream_format
@@ -674,6 +700,23 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
 
         return await self.send_setting(body)
 
+    async def set_backlight(self, value):
+        """Set the backlight parameter."""
+        if not self._isp_settings:
+            _LOGGER.error("Actual ISP settings not available")
+            return False
+
+        if value not in ["BackLightControl", "DynamicRangeControl", "Off"]:
+            _LOGGER.error("Invalid input: %s", value)
+            return False
+
+        new_value = value
+
+        body = [{"cmd": "SetIsp", "action": 0, "param": self._isp_settings["value"]}]
+        body[0]["param"]["Isp"]["backLight"] = new_value
+
+        return await self.send_setting(body)
+
     async def set_recording(self, enable):
         """Set the recording parameter."""
         if not self._recording_settings:
@@ -781,6 +824,76 @@ class Api:  # pylint: disable=too-many-instance-attributes disable=too-many-publ
             body[0]["param"]["id"] = preset
         return await self.send_setting(body)
 
+    async def send_search(
+        self, start: datetime, end: datetime, only_status: bool = False
+    ) -> Tuple[List[typings.SearchStatus], Optional[List[typings.SearchFile]]]:
+        """Send search command."""
+        body = [
+            {
+                "cmd": "Search",
+                "action": 0,
+                "param": {
+                    "Search": {
+                        "channel": self._channel,
+                        "onlyStatus": 0 if not only_status else 1,
+                        "streamType": self._stream,
+                        "StartTime": {
+                            "year": start.year,
+                            "mon": start.month,
+                            "day": start.day,
+                            "hour": start.hour,
+                            "min": start.minute,
+                            "sec": start.second,
+                        },
+                        "EndTime": {
+                            "year": end.year,
+                            "mon": end.month,
+                            "day": end.day,
+                            "hour": end.hour,
+                            "min": end.minute,
+                            "sec": end.second,
+                        },
+                    }
+                },
+            }
+        ]
+
+        command = body[0]["cmd"]
+        _LOGGER.debug(
+            "Sending command: %s to: %s with body: %s", command, self._host, body
+        )
+        response = await self.send(body, {"cmd": command})
+        if response is None:
+            return None, None
+
+        try:
+            json_data = json.loads(response)
+            _LOGGER.debug("Response from %s: %s", self._host, json_data)
+        except (TypeError, json.JSONDecodeError):
+            _LOGGER.debug(
+                "Host %s: Error translating %s response to json", self._host, command
+            )
+            return None, None
+        except KeyError as key_error:
+            _LOGGER.debug(
+                "Host %s: Received an unexpected response while sending command: %s, %s",
+                self._host,
+                command,
+                key_error,
+            )
+            return None, None
+
+        if json_data is not None:
+            if json_data[0]["code"] == 0:
+                search_result = json_data[0]["value"]["SearchResult"]
+                if only_status or "File" not in search_result:
+                    return search_result["Status"], None
+
+                return search_result["Status"], search_result["File"]
+
+        _LOGGER.debug("Host: %s: Failed to get results for %s", self._host, command)
+        return None, None
+
     async def send_setting(self, body):
         """Send a setting."""
         command = body[0]["cmd"]
